import math
import numpy as np  # Om data als arrays op te kunnen slaan en de normale verdeling te kunnen tekenen
import pandas as pd
from scipy.spatial import Voronoi
from scipy.stats import norm  # Om de lijn van de normale verdeling te tekenen. Getallen zijn zelf berekend.


def perc_overlap(mid_afstand: float, mu: float, sigma: float) -> float:
    """Berekend het percentage overlap tussen twee punten.
        norm.cdf(-n) berekend de kans tot n SD.
        In de wiskunde wordt vaak (1 - norm.cdf(n)) i.p.v. bovenstaande

        Door deze waarde maal twee te doen hebben we de totale overlapping.

       :param mid_afstand: Afstand van de twee punten tot het middelpunt
       :param mu: gemiddelde van de normale verdelingen
       :param sigma: standaard deviatie van de normale verdelingen
       :returns: het totale percentage dat de twee normale verdelingen elkaar overlappen.
                 0 < perc < 1
    """
    if not isinstance(mid_afstand, (int, float)) or not isinstance(mu, (int, float)) \
            or not isinstance(sigma, (int, float)):
        raise ValueError("Verkeerde waardes meegegeven als argumenten")

    if sigma <= 0:
        raise ValueError("Sigma kan niet negatief zijn")

    return norm.cdf(-mid_afstand + mu, loc=mu, scale=sigma) * 2


def perc_overlap_matrix(punten: np.ndarray, mu: float, sigma: float) -> np.ndarray:
    """
    :param punten: Numpy array van n bij 2. Iedere row bevat een x en y coördinaat.
    :param mu: gemiddelde van de normale verdelingen
    :param sigma: standaard deviatie van de normale verdelingen
    :returns: matrix van n bij n met alle percentages dat de punten overlappen
    """
    if not isinstance(punten, np.ndarray) or not isinstance(mu, (int, float)) \
            or not isinstance(sigma, (int, float)):
        raise ValueError("Verkeerde waardes meegegeven als argumenten")

    output_matrix = []

    for punt1 in punten:
        for punt2 in punten:
            middelpunt = abs((punt2 - punt1) / 2)  # [A, B]

            middelpunt_afstand = np.sqrt(  # Wortel van [A^2 + B^2]
                np.sum(  # ΔA^2 + ΔB^2
                    np.square(  # Macht van ΔA en ΔB
                        middelpunt)))  # [ΔA, ΔB]

            output_matrix.append(perc_overlap(middelpunt_afstand, mu, sigma))

    return np.array(output_matrix).reshape(len(punten), len(punten))


def grens(grens_perc: float, mu: float, sigma: float) -> float:
    """Berekend het aantal standaard deviaties dat nodig is om op een bepaalde cumulatieve kans te komen.
       Dezelfde functie als invNorm op de Texas Instruments grafische rekenmachines.
       0.00 < grens < 1.00
       Kan gebruikt worden om de hoogte van de Ellipse in Voronoi-diagrammen te bepalen

       :param grens_perc: getal tussen 0 en 1. Stel grens_waarde is 0.2 dan wordt het aantal standaard deviaties van mu(50%) tot 20% berekend.
       :param mu: gemiddelde van de normale verdelingen
       :param sigma: standaard deviatie van de normale verdelingen
       :return: het aantal standaard deviaties van mu tot de grens_waarde
    """
    if not isinstance(grens_perc, (int, float)) or not isinstance(mu, (int, float)) \
            or not isinstance(sigma, (int, float)):
        raise ValueError("Verkeerde waardes meegegeven als argumenten")

    if grens_perc >= 1 or grens_perc <= 0:
        raise ValueError("Grens kan niet meer dan 100%, negatief of 0 zijn")

    return norm.ppf(grens_perc, loc=mu, scale=sigma)


def helling(punt1: np.ndarray, punt2: np.ndarray) -> float:
    """Berekend de helling in graden tussen twee coordinaten.
       Kan gebruikt worden om de richting van de Ellipse in Voronoi-diagrammen te bepalen.

       :param punt1: Numpy array met een enkele row: [x, y]
       :param punt2: Numpy array met een enkele row: [x, y]
       :return: de hoek in aantal graden tussen punt1 en punt2
    """
    if not isinstance(punt1, np.ndarray) or not isinstance(punt2, np.ndarray):
        raise ValueError("Verkeerde waardes meegegeven als argumenten")

    delta = punt2 - punt1

    if delta[0] != 0.0:  # ZeroDivisionError voorkomen
        richtings_coefficient = delta[1] / delta[0]
        return math.degrees(math.atan(richtings_coefficient))
    else:
        return -90


def middelpunt_afstanden(punten: np.ndarray) -> list:
    """Berekend de afstand tot het middelpunt voor iedere combinatie van twee punten.
       Input is een np.array() van N bij 2.
       Gebaseerd op: https://github.com/scipy/scipy/blob/master/scipy/spatial/_plotutils.py

       :param punten: Numpy array van n bij 2. Iedere row bevat een x en y coördinaat.
       :return: lijst van afstanden tussen alle belanghebbende punten
    """
    if not isinstance(punten, (np.ndarray, pd.DataFrame)):
        raise ValueError("Verkeerde waardes meegegeven als argumenten")

    vor = Voronoi(punten)
    afstanden = []

    for punt in vor.ridge_points:
        middelpunt = vor.points[punt].mean(axis=0)

        mid_afstand = np.sqrt(  # Wortel van [A^2 + B^2]
            np.sum(  # ΔA^2 + ΔB^2
                np.square(  # Macht van ΔA en ΔB
                    middelpunt - vor.points[punt][0])))  # [ΔA, ΔB]

        afstanden.append(mid_afstand)
    return afstanden


def matrix_maal_matrix(m1: np.ndarray, m2: np.ndarray) -> np.ndarray:
    """Vermenigvuldigd matrix m maal n en zet alle waardes boven 1, op 1.
    Dit aangezien de 'infectiegraad' nooit meer dan 100% kan zijn.

    :param m1: matrix van n bij n
    :param m2: matrix van n bij n
    :return: matrix van n bij n, geen enkele waarde meer dan 1
    """
    if not isinstance(m1, np.ndarray) or not isinstance(m2, np.ndarray):
        raise ValueError("Verkeerde waardes meegegeven als argumenten")

    matrix = np.matmul(m1, m2)
    return np.where(matrix > 1, 1, matrix)


def perc_volledige_infectie(matrix_verloop: np.ndarray) -> tuple:
    """Berekend voor ieder punt in punten_matrix na hoeveel periodes deze volledig was geinfecteerd.
    Berekend vervolgens op hoeveel procent van de totale duur dat was.
    Iedere rij staat voor een punt. De lengte van de rij voor het aantal periodes.

    :param matrix_verloop: matrix van n bij m. n-aantal punten en m-aantal periodes

    :return: tuple met op hoeveel procent ieder punt volledig was geinfecteerd
    """
    if not isinstance(matrix_verloop, np.ndarray):
        raise ValueError("Verkeerde waardes meegegeven als argumenten")

    lijst = []

    for punt in matrix_verloop:
        voll_inf = np.where(punt == 1)  # Alle waardes waarbij het punt volledig is geinfecteerd

        try:
            lijst.append(voll_inf[0][0])  # Eerste waarde waarbij het punt volledig is geinfecteerd
        except IndexError:  # Als het punt nooit volledig is geinfecteerd
            if lijst:  # Als er al eerdere waardes zijn gegenereerd
                lijst.append(max(lijst))
            else:  # Als het de eerste waarde is
                lijst.append(1)

    return tuple((1 / max(lijst) * punt for punt in lijst))


def matrix_vec_verloop(punten: np.ndarray, vector: np.ndarray, perioden: int, mu: float = 0,
                       sigma: float = 1) -> np.ndarray:
    """Berekend voor iedere stap in de tijd de matrix uit. Iedere stap wordt opgeslagen en vormen samen
    een n bij m matrix. N voor het aantal punten en m voor het aantal periodes.

    :param punten: Numpy array van n bij 2. Iedere row bevat een x en y coördinaat.
    :param vector: Numpy array van n bij 1. Bevat per cell 0 of 1. Iedere cell waar 1 staat, begint de infectie.
    :param perioden:  Het aantal perioden dat het verloop berekend en getoont moet worden.
    :param mu: gemiddelde van de normale verdelingen
    :param sigma: standaard deviatie van de normale verdelingen

    :return: n bij m matrix met het verloop. Kan worden geplot
    """
    if not isinstance(punten, np.ndarray) or not isinstance(vector, np.ndarray) or not isinstance(perioden, int) \
            or not isinstance(mu, (float, int)) or not isinstance(sigma, (float, int)):
        raise ValueError("Verkeerde waardes meegegeven als argumenten")

    if punten.shape[0] != vector.shape[0]:
        raise ValueError("Vector en punten moeten even lang zijn.")

    matrix_verloop = []
    matrix_percentages = perc_overlap_matrix(punten, mu, sigma)

    for periode in range(perioden + 1):  # Reken voor iedere periode het matrix vector dotproduct uit
        i = 0
        matrix = matrix_percentages.copy()
        while i != periode:
            matrix = matrix_maal_matrix(matrix, matrix_percentages)
            i += 1
        matrix_verloop.append(vector.dot(matrix))

    return np.vstack(matrix_verloop).T  # Maakt van een lijst met lijsten een numpy array



import matplotlib.pyplot as plt  # Voor de grafieken
import numpy as np  # Om data als arrays op te kunnen slaan en de normale verdeling te kunnen tekenen
from matplotlib import style  # Style van de grafiek aanpassen naar eigen smaak
from scipy.spatial import Voronoi
from scipy.stats import norm  # Om de lijn van de normale verdeling te tekenen. Getallen zijn zelf berekend.

from plaguepy import bereken

style.use('seaborn')


def normale_verdeling(mu: float, sigma: float, mid_afstand: float, labels: tuple) -> None:
    """Plot-code afgeleid van: Thijs van den Berg (Jun. 2019)
       https://github.com/Denbergvanthijs/AC-opdrachten/

       Mu en sigma zijn vaak 0 resp. 1
       mid_afstand is de hemelsbrede afstand naar het middelpunt vanaf een punt

       :param mu: gemiddelde van de normale verdelingen
       :param sigma: standaard deviatie van de normale verdelingen
       :param mid_afstand: Afstand van de twee punten tot het middelpunt
       :param labels: Lijst van twee strings, labels[0] is de naam van eerste verdeling, idem voor labels[1]

       :returns: Plot met twee normale verdelingen. In de titel het percentage dat zij overlappen
                 en de meegegeven afstand tot het middelpunt.
    """
    if not isinstance(mu, (int, float)) or not isinstance(sigma, (int, float)) \
            or not isinstance(mid_afstand, (int, float)) or not isinstance(labels, (list, tuple)):
        raise ValueError("Verkeerde waardes meegegeven als argumenten")

    if len(labels) != 2:
        raise ValueError("Lijst/tuple `labels` moet twee elementen bevatten")

    plt.rcParams['figure.figsize'] = [9, 6]  # Grotere plots in Jupyter Notebook
    percentage_overlap = bereken.perc_overlap(mid_afstand, mu, sigma)
    lijn = np.linspace(mu - 4 * sigma - mid_afstand,
                       mu + 4 * sigma + mid_afstand)  # De lengte van de lijn van normale verdelingen

    plt.plot(lijn, norm.pdf(lijn, mu - mid_afstand, sigma), label=labels[0])  # De linker normale verdeling
    plt.plot(lijn, norm.pdf(lijn, mu + mid_afstand, sigma), label=labels[1])  # De rechter normale verdeling
    plt.axvline(x=mu, color="black", linestyle='--', label='Middelpunt')

    plt.xlim(mu - 3 * sigma - mid_afstand,
             mu + 3 * sigma + mid_afstand)  # Tenminste 99,8% zichtbaar van beide verdelingen
    plt.ylim(bottom=0)
    plt.xlabel("Aantal Standaard Deviaties")
    plt.ylabel("Kans")
    plt.title(f"{percentage_overlap * 100:.2f}% overlapt in totaal.\nAfstand tot het middelpunt: {mid_afstand:.2f} SD.")
    plt.legend()
    plt.tight_layout()  # Zodat ook de astitels op de grafieken passen
    plt.show()


def normale_verdeling_compleet(punten: np.ndarray) -> None:
    """Plot de normale verdelingen van alle mogelijke combinaties van punten.
        Een zichtbare Voronoi is niet nodig om berekeningen uit te voeren.
        De voronoi wordt echter wel gebruikt om de punten te berekenen.

        :param punten: Numpy array van n bij 2. Iedere row bevat een x en y coördinaat.
        :returns: Alle mogelijke plots met twee normale verdelingen.
                  In de titel het percentage dat zij overlappen
                  en de meegegeven afstand tot het middelpunt.
                  Normale verdelingen die minder dan 0.00% overlappen worden niet geplot.

    """
    if not isinstance(punten, np.ndarray):
        raise ValueError("Verkeerde waardes meegegeven als argumenten")

    vor = Voronoi(punten)
    mid_afstanden = bereken.middelpunt_afstanden(punten)

    for i, punt in enumerate(mid_afstanden):
        normale_verdeling(0, 1, punt, tuple(vor.points[vor.ridge_points[i]]))



import os
import time

import pandas as pd
from geopy.geocoders import Nominatim


def straat2coord(file_path: str, woonplaats: str, woonplaats_header: str, adres_header: str, sep: str = ";") -> None:
    """Berekend aan de hand van een CSV-bestand de breedte- en hoogtegraad.
       Resultaten worden opgeslagen in een nieuw CSV-bestand `data/geoDataKDV.csv`.
       Als input wordt om een woonplaats gevraagd. Alle punten die aan de waarde 'woonplaats voldoen'
       in de kolom 'woonplaatsHeader' worden geimporteerd.

       De breedte- en lengtegraad van de waardes die zich bevinden in de kolom 'adresHeader' worden opgevraagd.
       Duplicaten worden direct overgeslagen.

       :param file_path: totale path naar het te converteren bestand
       :param woonplaats: woonplaats waar een selectie uit (landelijke) data word gehaald
       :param woonplaats_header: kolom waar de waarde `woonplaats` zich in bevind
       :param adres_header: kolom met de adressen die omgezet mooeten worden. Idealieter adres + huisnummer
       :param sep: separator voor CSV-bestand, standaard ';' maar kan ook ',' of iets anders zijn

       :returns: Geconverteerd bestand opgeslagen in data/output/. Bestand bevat de headers latitude en longitude
    """
    if not isinstance(file_path, str) or not isinstance(woonplaats, str) \
            or not isinstance(woonplaats_header, str) or not isinstance(adres_header, str):
        raise ValueError("Verkeerde waardes meegegeven als argumenten")

    print("Even geduld a.u.b, dit kan even duren...")
    csv_data = pd.read_csv(file_path, sep=sep)  # Data uitlezen uit bestand
    subset = csv_data.loc[csv_data[woonplaats_header] == woonplaats]  # Selectie maken van de data

    geo_locator = Nominatim(user_agent="IPASS Project - Thijs van den Berg 2019")  # Variabele opzetten voor API-calls
    geo_locaties = pd.DataFrame(columns=['latitude', 'longitude'])  # DataFrame

    for adres in subset[adres_header].drop_duplicates():  # Ieder adres omzetten naar coördinaten
        try:
            locatie = geo_locator.geocode(f"{adres} {woonplaats}")  # Coordinaten opzoeken
        except GeocoderTimedOut:  # Te veel requests
            locatie = None

        if locatie is not None:
            geo_locaties = geo_locaties.append({'latitude': locatie.latitude, 'longitude': locatie.longitude},
                                               ignore_index=True)

        time.sleep(0.5)  # ToManyRequestsError tegengaan

    abs_path = os.path.basename(file_path)
    file_name = os.path.splitext(abs_path)[0]

    geo_locaties.to_csv(f"data/output/geo_{file_name}.csv", index=False)  # Data opslaan tbv de snelheid
    print(geo_locaties.head())


def coord2coord(file_path: str, lat_header: str, long_header: str) -> None:
    """Haalt uit een CSV-bestand de latitude- en longitudekolom.
       De gebruiker dient de namen van de kolommen waarde latitude en logitude
       zijn opgeslagen op te geven.
       Deze kolommen worden opgeslagen in een nieuw bestand met de prefix `geo_`.

       Het bestand kan vervolgens worden gebruikt om Voronoi's of kaarten te maken.

       :param file_path: totale path naar het te converteren bestand
       :param lat_header: naam van de kolom die de latitude bevat
       :param long_header: naam van de kolom die de longiutde bevat

       :returns: Geconverteerd bestand opgeslagen in data/output/. Bestand bevat de headers latitude en longitude
       """
    if not isinstance(file_path, str) or not isinstance(lat_header, str) \
            or not isinstance(long_header, str):
        raise ValueError("Verkeerde waardes meegegeven als argumenten")

    print("Even geduld a.u.b, dit kan even duren...")
    csv_data = pd.read_csv(file_path, sep=";")  # Data uitlezen uit bestand

    geo_locaties = csv_data[[lat_header, long_header]].rename(
        columns={lat_header: "latitude", long_header: "longitude"})  # Twee kolommen uit de data halen en hernoemen

    abs_path = os.path.basename(file_path)
    file_name = os.path.splitext(abs_path)[0]

    geo_locaties.to_csv(f"data/output/geo_{file_name}.csv", index=False)  # Data opslaan tbv de snelheid
    print(geo_locaties.head())



import cartopy.crs as ccrs
import cartopy.io.shapereader as shpreader
import matplotlib.lines as mlines
import matplotlib.pyplot as plt  # Voor de grafieken
import numpy as np  # Om data als arrays op te kunnen slaan en de normale verdeling te kunnen tekenen
import pandas as pd
from cartopy.io.img_tiles import Stamen
from matplotlib import cm
from matplotlib import style  # Style van de grafiek aanpassen naar eigen smaak
from matplotlib.patches import Ellipse
from scipy.spatial import Voronoi
from scipy.spatial import voronoi_plot_2d as SPvorPlot

from plaguepy import bereken

style.use('seaborn')


def voronoi(punten: np.ndarray, mu: float, sigma: float, tekst: bool = True, ellipse: bool = True) -> None:
    """Plot een Voronoi-diagram ter grote van de maximale x en y-coördinaten.
       Bij ieder middelpunt is eventueel het %-overlap te zien en het daarbijbehordende bereik.

       :param punten: Numpy array van n bij 2. Iedere row bevat een x en y coördinaat.
       :param mu: gemiddelde van de normale verdelingen
       :param sigma: standaard deviatie van de normale verdelingen
       :param tekst: keuze of de percentages zichtbaar zijn in de plot.
                     Aan te raden om uit te zetten bij veel punten.
       :param ellipse: keuze of de ellipsen zichtbaar zijn in de plot.
                     Aan te raden om uit te zetten bij veel punten

       :returns: Plot met een Voronoi-diagram en stippen op de coordinaten meegegeven in 'punten'
       """
    if not isinstance(punten, np.ndarray) or not isinstance(mu, (float, int)) or not isinstance(sigma, (float, int)) \
            or not isinstance(tekst, bool) or not isinstance(ellipse, bool):
        raise ValueError("Verkeerde waardes meegegeven als argumenten")

    plt.rcParams['figure.figsize'] = [8, 8]  # Grotere plots in Jupyter Notebook

    vor = Voronoi(punten)  # Voronoi berekenen
    SPvorPlot(vor)  # Voronoi plotten
    ax = plt.gca()

    if tekst or ellipse:
        mid_afstanden = bereken.middelpunt_afstanden(punten)  # Bereken alle middelpuntafstanden

        for i, punt in enumerate(vor.ridge_points):
            perc_overlap = bereken.perc_overlap(mid_afstanden[i], mu, sigma)
            middelpunt = vor.points[punt].mean(axis=0)

            if round(perc_overlap * 100, 2) > 0.00:  # Als er meer dan 0.00% overlap is, teken dan
                if ellipse:
                    graden = bereken.helling(vor.points[punt][0], vor.points[punt][1])
                    grens = bereken.grens(0.999, mu, sigma)

                    ax.add_artist(
                        Ellipse((middelpunt[0], middelpunt[1]), sigma * perc_overlap, grens - mid_afstanden[i],
                                angle=graden, color="red", fill=False))  # Ellipse tekenen
                if tekst:
                    plt.text(middelpunt[0], middelpunt[1], f"{round(perc_overlap * 100, 2)}%")  # Tekst tekenen

    plt.title(f"Aantal punten: {punten.shape[0]}")
    plt.xlabel("X-coördinaten")
    plt.ylabel("Y-coördinaten")

    if ellipse:
        legenda = (mlines.Line2D([0], [0], marker='o', color='w', label='Punt', markerfacecolor='b', markersize=15),
                   mlines.Line2D([0], [0], marker='o', color='w', label='Overlapping', markerfacecolor='r',
                                 markersize=15))
    else:
        legenda = [mlines.Line2D([0], [0], marker='o', color='w', label='Punt', markerfacecolor='b', markersize=15)]

    plt.legend(handles=legenda)
    plt.show()


def verloop(punten: np.ndarray, vector: np.ndarray, perioden: int, mu: float = 0, sigma: float = 1,
            legenda: bool = True, cmap_type: str = "hot") -> None:
    """Plot het verloop van een verspreiding waarbij het begin van de infectie
       in de vector wordt aangegeven. Verspreiding op tijdstip t gaat
       volgende de formule v·M^t.

       :param punten: Numpy array van n bij 2. Iedere row bevat een x en y coördinaat.
       :param vector: Numpy array van n bij 1. Bevat per cell 0 of 1.
                      Iedere cell waar 1 staat, begint de infectie.
       :param perioden:  Het aantal perioden dat het verloop berekend en getoont moet worden.
       :param mu: gemiddelde van de normale verdelingen
       :param sigma: standaard deviatie van de normale verdelingen
       :param legenda: optioneel, toont de legenda.
              Aangeraden om uit te zetten als er heel veel punten dienen te worden geplot
       :param cmap_type: De te gebruiken colormap voor de plot

       :returns: Plot met het verloop van de infectiegraad/verspreiding van de punten
    """
    if not isinstance(punten, np.ndarray) or not isinstance(vector, np.ndarray) or not isinstance(perioden, int) \
            or not isinstance(mu, (float, int)) or not isinstance(sigma, (float, int)) or not isinstance(legenda, bool) \
            or not isinstance(cmap_type, str):
        raise ValueError("Verkeerde waardes meegegeven als argumenten")

    if punten.shape[0] != vector.shape[0]:
        raise ValueError("Vector en punten moeten even lang zijn.")

    print("Even geduld a.u.b, dit kan even duren...")
    plt.rcParams['figure.figsize'] = [8, 8]  # Grotere plots in Jupyter Notebook

    matrix_verloop = bereken.matrix_vec_verloop(punten, vector, perioden, mu=mu, sigma=sigma)
    perc_voll_inf = bereken.perc_volledige_infectie(matrix_verloop)

    color_map = cm.get_cmap(cmap_type, 12)
    for i, periode in enumerate(matrix_verloop):
        plt.plot(periode, color=color_map(perc_voll_inf[i]), label=f"Punt {punten[i]}")
        # Plot de infectiegraad van ieder punt tijdens iedere periode

    if legenda:
        plt.legend(loc="lower right")

    plt.xlabel("Periodes")
    plt.ylabel("Infectiegraad")
    plt.xlim(0, perioden)
    plt.ylim(0, 1.1)
    plt.title(f"Aantal punten: {punten.shape[0]}")
    plt.tight_layout()  # Alles past zo beter op de grafiek
    plt.show()


def kaart(file_path: str, punten: np.ndarray, vector: np.ndarray, perioden: int, terrein: bool = True, sep: str = ",",
          mu: float = 0, sigma: float = 1, cmap_type: str = "Dark2") -> None:
    """Maakt een kaart gebaseerd op de coordinaten in een CSV-bestand.

    :param file_path: Pad naar het te plotten CSV-bestand
    :param punten: Numpy array van n bij 2. Iedere row bevat een x en y coördinaat.
    :param vector: Numpy array van n bij 1. Bevat per cell 0 of 1. Iedere cell waar 1 staat, begint de infectie.
    :param perioden:  Het aantal perioden dat de kaart moet berekenen.
    :param terrein: optioneel, zorgt voor een grafische achtergrond van de plot. Kan enkele seconden langer duren.
    :param sep: optioneel, seperator van het CSV-bestand
    :param mu: gemiddelde van de normale verdelingen
    :param sigma: standaard deviatie van de normale verdelingen
    :param cmap_type: De te gebruiken colormap voor de plot

    :returns: Een kaart met de geplotte punten, al dan niet met een 'terrein-achtergrond'. Kleur van de stippen staat voor de periode wanneer zij volledig besmet raakten.
    """
    if not isinstance(file_path, str) or not isinstance(punten, np.ndarray) or not isinstance(vector, np.ndarray) \
            or not isinstance(perioden, int) or not isinstance(terrein, bool) or not isinstance(sep, str) or not \
            isinstance(mu, (float, int)) or not isinstance(sigma, (float, int)) or not isinstance(cmap_type, str):
        raise ValueError("Verkeerde waardes meegegeven als argumenten")

    if punten.shape[0] != vector.shape[0]:
        raise ValueError("Vector en punten moeten even lang zijn.")

    print("Kaart aan het maken. \nEven geduld a.u.b, dit kan even duren...")
    plt.rcParams['figure.figsize'] = [8, 8]  # Grotere plots in Jupyter Notebook

    coordinaten = pd.read_csv(file_path, sep=sep)
    coordinaten = coordinaten.loc[(coordinaten['latitude'] < 53.5) & (coordinaten['latitude'] > 50.7) &
                                  (coordinaten['longitude'] < 7.3) & (coordinaten['longitude'] > 3.3)]  # Filter NL
    coordinaten = coordinaten.values[:, :]  # DataFrame omzetten naar Numpy-array

    sf_path = 'data/shapefiles/gadm36_NLD_2.shp'
    sf_data = list(shpreader.Reader(sf_path).geometries())
    ax = plt.axes(projection=ccrs.EuroPP())

    if terrein:
        ax.add_image(Stamen('terrain-background'), 12)
        ax.add_geometries(sf_data, ccrs.PlateCarree(), edgecolor='black', facecolor='none', alpha=1)
    else:
        ax.add_geometries(sf_data, ccrs.PlateCarree(), edgecolor='black', facecolor='orange', alpha=0.2)

    ax.set_extent([min(coordinaten[:, 1]), max(coordinaten[:, 1]),
                   min(coordinaten[:, 0]), max(coordinaten[:, 0])])  # Grootte gelijk aan min/max van coordinaten

    matrix_verloop = bereken.matrix_vec_verloop(punten, vector, perioden, mu=mu, sigma=sigma)
    perc_voll_inf = bereken.perc_volledige_infectie(matrix_verloop)
    color_map = cm.get_cmap(cmap_type, 12)

    for i, coord in enumerate(coordinaten):  # Stippen tekenen
        ax.plot(coord[1], coord[0], marker='o', markersize=3, color=color_map(perc_voll_inf[i]),
                transform=ccrs.PlateCarree())

    for i, punt in enumerate(vector):
        if punt:
            ax.plot(punten[i][1], punten[i][0], marker='D', markersize=12, color="red", transform=ccrs.PlateCarree())

    plt.show()



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name='plaguepy',
    version="2019.07.2.1",
    author="Thijs van den Berg",
    author_email="denbergvanthijs@gmail.com",
    description='Ziekteverspreiding gevisualiseerd',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/denbergvanthijs/plaguepy",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
)



import numpy as np
import pytest

from plaguepy import bereken


def test_perc_overlap():
    assert bereken.perc_overlap(0, 0, 1) == 1.0
    assert bereken.perc_overlap(0, 1, 1) == 1.0
    assert bereken.perc_overlap(1, 0, 1) == pytest.approx(0.3173105)
    assert bereken.perc_overlap(1, 1, 1) == pytest.approx(0.3173105)

    with pytest.raises(ValueError):
        assert bereken.perc_overlap(0, 1, 0) == ValueError
        assert bereken.perc_overlap(1, 0, 0) == ValueError
        assert bereken.perc_overlap(0, 0, 0) == ValueError
        assert bereken.perc_overlap(1, 1, 0) == ValueError


def test_grens():
    assert bereken.grens(0.025, 0, 1) == pytest.approx(-1.95996398)  # -2 SD
    assert bereken.grens(0.16, 0, 1) == pytest.approx(-0.99445788)  # -1 SD
    assert bereken.grens(0.5, 0, 1) == 0.0  # 50% dus nog op gemiddelde
    assert bereken.grens(0.84, 0, 1) == pytest.approx(0.99445788)  # 1 SD
    assert bereken.grens(0.975, 0, 1) == pytest.approx(1.95996398)  # 2 SD

    with pytest.raises(ValueError):
        assert bereken.grens(-0.5, 0, 1) == ValueError
        assert bereken.grens(0, 0, 1) == ValueError
        assert bereken.grens(1, 0, 1) == ValueError
        assert bereken.grens(1.5, 0, 1) == ValueError


def test_helling():
    assert bereken.helling(np.array([0, 0]), np.array([1, 0])) == 0.0
    assert bereken.helling(np.array([0, 0]), np.array([1, 1])) == 45.0
    assert bereken.helling(np.array([0, 0]), np.array([-1, -1])) == 45.0

    assert bereken.helling(np.array([0, 0]), np.array([-1, 1])) == -45.0
    assert bereken.helling(np.array([0, 0]), np.array([1, -1])) == -45.0

    assert bereken.helling(np.array([0, 0]), np.array([0, 0])) == -90
    assert bereken.helling(np.array([0, 0]), np.array([0, 1])) == -90
